// -----------------------------------------------------------------------
//	Stroboscope
//	K15 2011
// -----------------------------------------------------------------------

/* ----------------------------------------------------------------------- */

#include <htc.h>
#include <lcd.h>
#include <delay.h>

#define CLRWDT		asm("clrwdt")
/* ----------------------------------------------------------------------- */

__CONFIG(FOSC_HS);      // crystal 20 MHz

char display[6];
unsigned long rpm;
unsigned short counter;
unsigned short timer;

char mode;             // 2T x 4T
#define MODE_2T         1    
#define MODE_4T         2

#define INPUT           RA4
#define LED             RA3

//#define TC              9       // interrupt duration in counter ticks (without LED delay)
#define TC              175       // interrupt duration in counter ticks (without LED delay)
//#define TQ              9692290   // delay = counter/16 "magic number" :-] - divided by ticks, gives rpm
#define TQ              9927234     // delay = counter/64

void interrupt tick(void) {

    LED = 1;
    delay(counter / 64);        // 1 = ~3.4 us
    LED = 0;

    delay(294);                 // ~ 1 ms

    rpm = counter;
    counter = TC;

    TMR0 = 0xFF;
    T0IF = 0;
}

void main() {

    TRISA = 0x10;		// port A - RA4 input, the rest outputs
    TRISB = 0x00;		// LCD display data output

    T0IE = 1;		// enable timer/counter interrupt
    T0CS = 1;		// switch Timer0 to counter mode
    T0SE = 1;		// increment on falling edge
    PSA  = 1;		// do not use prescaller with Timer0 (use it with watchdog)
    OPTION_REGbits.PS = 7;  // WDT prescaler to 128 - lazy dog :-)
    TMR0 = 0xFF;		// overflow with the first pulse

    counter = 0;
    timer = 0;

    LED = 0;

    mode = MODE_2T;

    init_lcd();

    write_lcd_data('S');
    write_lcd_data('t');
    write_lcd_data('r');
    write_lcd_data('o');
    write_lcd_data('b');
    write_lcd_data('o');
    write_lcd_data('s');
    write_lcd_data('k');
    write_lcd_cmd(0xC0);
    write_lcd_data('o');
    write_lcd_data('p');
    write_lcd_data(' ');
    write_lcd_data('-');
    write_lcd_data(' ');
    write_lcd_data('K');
    write_lcd_data('1');
    write_lcd_data('5');

    while (++counter) CLRWDT;
    while (++counter) CLRWDT;
    while (++counter) CLRWDT;
    while (++counter) CLRWDT;
    
    GIE = 1;	// enable interrupts

    while (1) {
        CLRWDT;
        // 1 tick of counter = ~6.4 us
        if (!counter++) {
            rpm = 0;

            // if the INPUT is low for more than ~300 ms, switch the mode
            if (!INPUT) {
                write_lcd_cmd(0x80); 	// line 1
                switch (mode) {
                    case MODE_2T:
                        mode = MODE_4T; 
                        write_lcd_data('4');
                    break;
                    case MODE_4T: 
                        mode = MODE_2T;
                        write_lcd_data('2');
                    break;
                }
                // wait till the button release...
                // reset when the button is held down too long
                while (!INPUT) ;
            }
        }

        // update display every ~300ms
        if (!timer++) {
            unsigned long n = rpm;

            n = TQ / n;
            n *= mode;

            char *p = display + 6;
            do {
                    *--p = n % 10 + '0';
                    n /= 10;
            } while (n != 0);

            while (display < p)
                *--p = ' ';

            write_lcd_cmd(0x80); 	// line 1
            switch (mode) {
                case MODE_4T: write_lcd_data('4'); break;
                case MODE_2T: write_lcd_data('2'); break;
            }
            write_lcd_data('-');
            write_lcd_data('T');
            write_lcd_data(' ');
            write_lcd_data(' ');
            write_lcd_data(' ');
            write_lcd_data(display[0]);
            write_lcd_data(display[1]);
            write_lcd_cmd(0xC0);
            write_lcd_data(display[2]);
            write_lcd_data(display[3]);
            write_lcd_data(display[4]);
            write_lcd_data(display[5]);
            write_lcd_data(' ');
            write_lcd_data('r');
            write_lcd_data('p');
            write_lcd_data('m');
        }
    }
}

